//---------------------------------------------------------------------------
// Copyright (C) 1999 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//---------------------------------------------------------------------------
//
//  ibsha.h - Include file for SHA iButton scripts.  Contains
//             script step contants and scripts.
//
//  Version: 1.02
//

// output level contants
#define LV_ALWAYS                   2
#define LV_OPTIONAL                 1
#define LV_VERBOSE                  0

// SHA iButton commands
#define SHA_COMPUTE_FIRST_SCRT      0x0F
#define SHA_COMPUTE_NEXT_SCRT       0xF0
#define SHA_VALIDATE_PAGE           0x3C
#define SHA_SIGN_PAGE               0xC3
#define SHA_COMPUTE_CHALLENGE       0xCC
#define SHA_AUTHENTICATE_HOST       0xAA
#define CMD_READ_MEMORY             0xF0
#define CMD_MATCH_SCRATCH           0x3C
#define CMD_WRITE_SCRATCH           0x0F
#define CMD_READ_SCRATCH            0xAA
#define CMD_ERASE_SCRATCH           0xC3
#define CMD_COPY_SCRATCH            0x55
#define CMD_READ_AUTH_PAGE          0xA5
#define CMD_COMPUTE_SHA             0x33
// other
#define IB_ROVING                   0
#define IB_COPR                     1 
#define SHA_FAM                     0x18
// exit codes
#define EXIT_SUCCESS                0
#define EXIT_ERROR                  1
#define EXIT_ERROR_WRITE            2
#define EXIT_ERROR_MONEY_SCRT       3
#define EXIT_ERROR_AUTH_SCRT        4
#define EXIT_ERROR_NO_SHA           5

// basic types
typedef unsigned char  uchar;
typedef unsigned short ushort;
typedef unsigned long  ulong;

// certificate 
typedef struct
{
   ushort obj_descriptor;   // descriptor of object 0x0501
   uchar  obj_len;          // object length
   
   uchar  pkt_len;          // packet length
   ushort trans_id;         // transaction id
   ushort code_mult;        // monetary unit code & multiplier
   uchar  balance[3];       // monetary balance (should be 3 bytes)
   uchar  money_mac[20];    // monetary mac
   uchar  cert_type;        // certificate type & algorithm
   uchar  pkt_cont_ptr;     // packet continuation pointer
   uchar  inv_crc[2];
    
   ulong  write_cycle_cnt;  // write cycle counter
   uchar  money_page;       // monetary page
   uchar  serial_num[7];    // ibutton serial number (no CRC8)

   char   file_name[4];     // file name of account
   uchar  file_ext;         // file extension of account
   uchar  rom_id[8];        // device rom ID
   uchar  auth_mac[20];     // authentication mac

   uchar  constant_data[20];// system constant data
   char   user_data[29];    // fixed user data

} eCertificate;

// structure to hold each state in the StateMachine
typedef struct
{
   int  Step;
   char StepDescription[50];
} Script;


// step constants
enum { ST_SETUP=0, ST_CHECK_COPROC, ST_CHECK_ROVING, ST_ERASE_SCRATCH, 
       ST_GET_SESSION, ST_REL_SESSION, ST_VERIFY, ST_FINISH, 
       ST_READ_SCRATCH, ST_WRITE_SCRATCH, ST_COPY_SCRATCH, ST_READ_AUTH_PAGE,
       ST_TARGET_C_MMS, ST_TARGET_C_AMS, ST_TARGET_C_UDS, ST_TARGET_R_UDS,
       ST_TARGET_T_UDS, ST_TARGET_PAGE, ST_TARGET_SECRET, ST_SET_BUFF_ACCNT,
       ST_SET_BUFF_FF, ST_COMPUTE_SHA, ST_SHA_COMPUTE_FIRST_SCRT,
       ST_SHA_COMPUTE_NEXT_SCRT, ST_SHA_VALIDATE_PAGE, ST_SET_BUFF_MONEY,
       ST_SHA_SIGN_PAGE, ST_SHA_COMPUTE_CHALLENGE, ST_INC_COUNTER,
       ST_SHA_AUTHENTICATE_HOST, ST_SET_BUFF_ROM, ST_SELECT_ROVING,
       ST_SELECT_COPR, ST_SET_BUFF_00, ST_SET_BUFF_RAND, ST_SET_RAND_NULL,
       ST_GET_NEW_RAND, ST_SET_BUFF_VAL, ST_SAVE_MONEY_MAC, ST_SAVE_AUTH_MAC,
       ST_SET_BUFF_ACCNT_FXT, ST_MATCH_SCRATCH_MONEY, ST_MATCH_SCRATCH_AUTH,
       ST_PRESENT,ST_MAKE_ECERT, ST_BUF_INPUT, ST_TARGET_DIRECTORY, 
       ST_FORMAT_UDP, ST_SEARCH_TARGET, ST_FIND_SHA };

// status contants
enum { STATUS_STEP_COMPLETE, STATUS_COMPLETE, STATUS_INPROGRESS,
       STATUS_ERROR_HALT, STATUS_ERROR_TRANSIENT };

#ifdef TMEXUTIL

// verbose output mode
int VERBOSE=0;
// file for output
FILE *fp;
// 1-wire session handles for two ports
long session_handle[2];
// 1-wire port names and types for co-processor/roving
short PortNum[2],PortType[2];
// 1-wire state buffers co-processor/roving
unsigned char state_buffer[2][5120];
// current co-processor(1)/roving(0)
int dev=0;
// secret pages
int c_mmaster_scrt, c_amaster_scrt, c_udevice_scrt;
int r_udevice_scrt, t_udevice_scrt; 
// co-processor information filename
uchar copr_file[] = { 'C', 'O', 'P', 'R', 0 };
// co-processor rom number
short copr_rom[] = { 0,0,0,0,0,0,0,0 };
// provider file name (on roving)
uchar provider[] = { 0,0,0,0,0 };
// user information 
char user_data[255];

// Verify 1-Wire Net is OK
Script Verify1Wire[] = 
    {{ ST_SELECT_ROVING, "Select ROVING port" },
     { ST_GET_SESSION,   "Aquire a session" }, 
     { ST_SETUP,         "Verify 1-Wire"},
     { ST_REL_SESSION,   "Release the session" },
     { ST_SELECT_COPR,   "Select CO-PROCESSOR port" },
     { ST_GET_SESSION,   "Aquire a session" }, 
     { ST_SETUP,         "Verify 1-Wire"},
     { ST_REL_SESSION,   "Release the session" },     
     { ST_FINISH,        "Finished"}}; 

// search to find a roving token
Script FindRoving[] = 
    {{ ST_SELECT_ROVING,"Select ROVING port" },
     { ST_SEARCH_TARGET,"Target search for SHA" },
     { ST_FIND_SHA,     "Search for SHA" },
     { ST_CHECK_ROVING, "Check if device found is roving token" },
     { ST_FIND_SHA,     "Search for SHA" },
     { ST_CHECK_ROVING, "Check if device found is roving token" },
     { ST_FINISH,       "Finished"}}; 

// search to find the first SHA device
Script FindSHA[] = 
    {{ ST_SEARCH_TARGET,"Target search for SHA" },
     { ST_FIND_SHA,     "Search for SHA" },
     { ST_FINISH,       "Finished"}}; 

// verify presence of roving 
Script RovingIsPresent[] = 
    {{ ST_SELECT_ROVING,"Select ROVING port" },
     { ST_PRESENT,      "Verify device present" },
     { ST_FINISH,       "Finished"}}; 

// search to find a token
Script FindCoprocessor[] = 
    {{ ST_SELECT_COPR,  "Select CO-PROCESSOR port" },
     { ST_SEARCH_TARGET,"Target search for SHA" },
     { ST_FIND_SHA,     "Search for SHA" },
     { ST_CHECK_COPROC, "Check if device found is co-processor token" },
     { ST_FIND_SHA,     "Search for SHA" },
     { ST_CHECK_COPROC, "Check if device found is co-processor token" },
     { ST_FINISH,       "Finished"}}; 

// create the auth secret for co-processor
Script CreateAuthSecret[] = 
    {{ ST_SELECT_COPR,    "C_Select CO-PROCESSOR port" },
     { ST_BUF_INPUT,      "C_Put auth secret in temp buffer" },
     // write auth secret to page
     { ST_TARGET_C_AMS,   "C_Target co-processor auth master secret" },
     { ST_TARGET_PAGE,    "C_Target page of secret" }, 
     { ST_ERASE_SCRATCH,  "C_Erase the scratchpad of the co-processor device" }, 
     { ST_WRITE_SCRATCH,  "C_Write the scratchpad" },  
     { ST_COPY_SCRATCH,   "C_Copy the scratchpad" }, 
     // write FF's to scratchpad 
     { ST_SET_BUFF_FF,    "C_Set buffer to FF's" },
     { ST_ERASE_SCRATCH,  "C_Erase the scratchpad of the co-processor device" }, 
     { ST_WRITE_SCRATCH,  "C_Write the scratchpad" },  
     // compute first secret and copy to secret location
     { ST_SHA_COMPUTE_FIRST_SCRT, "C_Set SHA to compute first secret" },
     { ST_COMPUTE_SHA,    "C_Compute SHA" },
     { ST_TARGET_C_AMS,   "C_Target co-processor auth master secret" },
     { ST_TARGET_SECRET,  "C_Target secret" }, 
     { ST_WRITE_SCRATCH,  "C_Write the scratchpad (set address)" },  
     { ST_COPY_SCRATCH,   "C_Copy the scratchpad" }, 
     { ST_FINISH,         "Finished"}}; 

// create the money secret for co-processor
Script CreateMoneySecret[] = 
    {{ ST_SELECT_COPR,    "C_Select CO-PROCESSOR port" },
     { ST_BUF_INPUT,      "C_Put money secret in temp buffer" },
     // write money secret to page
     { ST_TARGET_C_MMS,   "C_Target co-processor money master secret" },
     { ST_TARGET_PAGE,    "C_Target page of secret" }, 
     { ST_ERASE_SCRATCH,  "C_Erase the scratchpad of the roving device" }, 
     { ST_WRITE_SCRATCH,  "C_Write the scratchpad" },  
     { ST_COPY_SCRATCH,   "C_Copy the scratchpad" }, 
     // write FF's into scratchpad
     { ST_SET_BUFF_FF,    "C_Set buffer to FF's" },
     { ST_ERASE_SCRATCH,  "C_Erase the scratchpad of the co-processor device" }, 
     { ST_WRITE_SCRATCH,  "C_Write the scratchpad" },
     // compute first secret and copy to secret location  
     { ST_SHA_COMPUTE_FIRST_SCRT, "C_Set SHA to compute first secret" },
     { ST_COMPUTE_SHA,    "C_Compute SHA" },
     { ST_TARGET_C_MMS,   "C_Target co-processor money master secret" },
     { ST_TARGET_SECRET,  "C_Target secret" }, 
     { ST_WRITE_SCRATCH,  "C_Write the scratchpad (set address)" },  
     { ST_COPY_SCRATCH,   "C_Copy the scratchpad" },
     { ST_FINISH,         "Finished"}}; 

// create the unique device secret for roaming ibutton
Script CreateUniqueDeviceSecret[] = 
    {{ ST_SELECT_ROVING,  "R_Select ROVING port" },
     // write auth secret to page
     { ST_BUF_INPUT,      "R_Put auth secret in temp buffer" },
     { ST_TARGET_R_UDS,   "R_Target roving unique secret" },
     { ST_TARGET_PAGE,    "R_Target page of secret" }, 
     { ST_ERASE_SCRATCH,  "R_Erase the scratchpad of the roving device" }, 
     { ST_WRITE_SCRATCH,  "R_Write the scratchpad" },  
     { ST_COPY_SCRATCH,   "R_Copy the scratchpad" }, 
     // put constant (FF's) in the scratchpad
     { ST_SET_BUFF_FF,    "R_Set buffer to FF's" },
     { ST_ERASE_SCRATCH,  "R_Erase the scratchpad of the roving device" }, 
     { ST_WRITE_SCRATCH,  "R_Write the scratchpad" },  
     // compute the first secret and copy to secret location (auth secret)
     { ST_SHA_COMPUTE_FIRST_SCRT, "R_Set SHA to compute first secret" },
     { ST_COMPUTE_SHA,    "R_Compute SHA" },
     { ST_TARGET_R_UDS,   "R_Target roving unique secret" },
     { ST_TARGET_SECRET,  "R_Target secret" }, 
     { ST_WRITE_SCRATCH,  "R_Write the scratchpad (set address)" },  
     { ST_COPY_SCRATCH,   "R_Copy the scratchpad" }, 
     // put contant (FF's) in page
     { ST_SET_BUFF_FF,    "R_Set buffer to FF's" },
     { ST_TARGET_R_UDS,   "R_Target roving unique secret" },
     { ST_TARGET_PAGE,    "R_Target page of secret" }, 
     { ST_ERASE_SCRATCH,  "R_Erase the scratchpad of the roving device" }, 
     { ST_WRITE_SCRATCH,  "R_Write the scratchpad" },  
     { ST_COPY_SCRATCH,   "R_Copy the scratchpad" }, 
     // put ROM in scratchpad
     { ST_TARGET_R_UDS,   "R_Target roving unique secret" },
     { ST_TARGET_PAGE,    "R_Target page of secret" }, 
     { ST_ERASE_SCRATCH,  "R_Erase the scratchpad of the roving device" }, 
     { ST_SET_BUFF_ROM,   "R_Move ROM to temporary buffer" },
     { ST_WRITE_SCRATCH,  "R_Write the scratchpad" },
     // compute the next secret
     { ST_SHA_COMPUTE_NEXT_SCRT, "R_Set SHA to compute next secret" },
     { ST_COMPUTE_SHA,    "R_Compute SHA" },
     // copy to unique data secret
     { ST_SET_BUFF_FF,    "R_Set buffer to FF's" },
     { ST_TARGET_R_UDS,   "R_Target roving unique device secret" },
     { ST_TARGET_SECRET,  "R_Target secret" }, 
     { ST_WRITE_SCRATCH,  "R_Write the scratchpad (set address)" },  
     { ST_COPY_SCRATCH,   "R_Copy the scratchpad" }, 
     { ST_FINISH,         "Finished"}}; 

// Select the coprocessor 
Script SelectCoProcessor[] =
    {{ ST_SELECT_COPR,    "C_Select CO-PROCESSOR port" },
     { ST_FINISH,         "Finished"}}; 
     
// Select the roving
Script SelectRoving[] =
    {{ ST_SELECT_ROVING,  "R_Select ROVING port" },
     { ST_FINISH,         "Finished"}}; 

// write a page specified to the current device
Script WritePage[] =
    {{ ST_BUF_INPUT,      "Put data to write in temp buffer" },
     { ST_FORMAT_UDP,     "Format data in temp buf to UDP" },
     { ST_ERASE_SCRATCH,  "Erase the scratchpad of the co-processor device" }, 
     { ST_WRITE_SCRATCH,  "Write the scratchpad" },  
     { ST_COPY_SCRATCH,   "Copy the scratchpad" },
     { ST_FINISH,         "Finished"}}; 
      
// open the sessions
Script OpenSession[] = 
    {// session for token
     { ST_SELECT_ROVING,  "Select ROVING port" },
     { ST_GET_SESSION,    "Aquire a session" }, 
     // session for coprocessor
     { ST_SELECT_COPR,    "Select CO-PROCESSOR port" },
     { ST_GET_SESSION,    "Aquire a session" }, 
     { ST_FINISH,         "Finished"}}; 

// open the sessions
Script CloseSession[] = 
    {// session for token
     { ST_SELECT_COPR,    "C_Select CO-PROCESSOR port" },
     { ST_REL_SESSION,    "Release the session" },
     // release the sessions       
     { ST_SELECT_ROVING,  "Select ROVING port" },
     { ST_REL_SESSION,    "Release the session" },
     { ST_FINISH,         "Finished"}}; 

// do a authenticated read of the roving monitary page
Script ReadAuth[] = 
    {// write 00 to co-processor auth master secret page
     { ST_SELECT_COPR,    "C_Select CO-PROCESSOR port" },
     { ST_ERASE_SCRATCH,  "C_Erase the scratchpad" }, 
     { ST_SET_BUFF_FF,    "C_Set buffer to FF's" },
     { ST_TARGET_C_AMS,   "C_Target co-processor auth master secret" },
     { ST_TARGET_PAGE,    "C_Target page of secret" }, 
     { ST_WRITE_SCRATCH,  "C_Write the scratchpad" },  
     { ST_COPY_SCRATCH,   "C_Copy the scratchpad" }, 
     // write token ROM to coprocessor scratch
     { ST_SELECT_ROVING,  "R_Select ROVING port" },
     { ST_SET_BUFF_ROM,   "R_Move ROM to temporary buffer" },
     { ST_SELECT_COPR,    "C_Select CO-PROCESSOR port" },
     { ST_WRITE_SCRATCH,  "C_Write the scratchpad" },  
     // compute device secret and save in co-processor unique device secret
     { ST_TARGET_C_AMS,   "C_Target co-processor auth master secret" },
     { ST_TARGET_PAGE,    "C_Target page of secret" }, 
     { ST_SHA_COMPUTE_NEXT_SCRT, "C_Set SHA to compute next secret" },
     { ST_COMPUTE_SHA,    "C_Compute SHA" },
     { ST_TARGET_C_UDS,   "C_Target co-processor unique device secret" },
     { ST_TARGET_SECRET,  "C_Target secret" }, 
     { ST_WRITE_SCRATCH,  "C_Write the scratchpad (set address)" },  
     { ST_COPY_SCRATCH,   "C_Copy the scratchpad" }, 
     // write random challenge to scratch of target
     { ST_SELECT_ROVING,  "R_Select ROVING port" },
     { ST_ERASE_SCRATCH,  "R_Erase the scratchpad" }, 
     { ST_TARGET_R_UDS,   "R_Target roving unique device secret" },
     { ST_TARGET_PAGE,    "R_Target page of secret" }, 
     { ST_GET_NEW_RAND,   "R_Get new random value" },
     { ST_SET_BUFF_RAND,  "R_Move random to temporary buffer" },
     { ST_WRITE_SCRATCH,  "R_Write the scratchpad" },  
     // read data page to check       
     { ST_TARGET_R_UDS,   "R_Target roving unique device secret" },
     { ST_TARGET_PAGE,    "R_Target page of secret" }, 
     { ST_READ_AUTH_PAGE, "R_Read Authenticate page" },
     { ST_MAKE_ECERT,     "R_Make eCert from read auth data" },
     // write data page to (unique secret) page of coprocessor
     { ST_SELECT_COPR,    "C_Select CO-PROCESSOR port" },
     { ST_ERASE_SCRATCH,  "C_Erase the scratchpad" }, 
     { ST_TARGET_C_UDS,   "C_Target co-processor unique device secret" },
     { ST_TARGET_PAGE,    "C_Target page of secret" }, 
     { ST_WRITE_SCRATCH,  "C_Write the scratchpad" },  
     { ST_COPY_SCRATCH,   "C_Copy the scratchpad" }, 
     // build the temp buffer with counter / rom / challenge
     { ST_SELECT_ROVING,  "R_Select ROVING port" },
     { ST_TARGET_R_UDS,   "R_Target roving unique device secret" },
     { ST_TARGET_PAGE,    "R_Target page of secret" }, 
     { ST_SET_BUFF_VAL,   "R_Create the validate data in temporary buffer" },
     // write validate to scratch of coprocessor
     { ST_SELECT_COPR,    "C_Select CO-PROCESSOR port" },
     { ST_TARGET_C_UDS,   "C_Target co-processor unique device secret" },
     { ST_TARGET_PAGE,    "C_Target page of secret" }, 
     { ST_WRITE_SCRATCH,  "C_Write the scratchpad" },  
     { ST_SHA_VALIDATE_PAGE, "C_Set SHA to validate page" },
     { ST_COMPUTE_SHA,    "C_Compute SHA" },
     // read scratch from token
     { ST_SELECT_ROVING,  "R_Select ROVING port" },
     { ST_READ_SCRATCH,   "R_Read the scratchpad" },  
     { ST_SAVE_AUTH_MAC,  "R_Save the authenticate MAC" },
     // do match scratchpad    
     { ST_SELECT_COPR,    "C_Select CO-PROCESSOR port" },
     { ST_MATCH_SCRATCH_AUTH,"C_Perform the match scratchpad on auth mac" },  
     { ST_FINISH,         "Finished"}}; 

// do a authenticated read of the roving monitary page
Script WriteMoney[] = 
    {// create packet for signing
     { ST_SELECT_COPR,    "C_Select CO-PROCESSOR port" },
     { ST_SET_BUFF_ACCNT_FXT, "C_Set (fxt) account in temporary buffer" },
     // write to money page of co-processor
     { ST_TARGET_C_MMS,   "C_Target co-processor money master secret" },
     { ST_TARGET_PAGE,    "C_Target page of secret" }, 
     { ST_ERASE_SCRATCH,  "C_Erase the scratchpad" }, 
     { ST_WRITE_SCRATCH,  "C_Write the scratchpad" },  
     { ST_COPY_SCRATCH,   "C_Copy the scratchpad" }, 
     // read the roving money page data and counters
     { ST_SELECT_ROVING,  "R_Select ROVING port" },
     { ST_TARGET_R_UDS,   "R_Target roving unique device secret" },
     { ST_TARGET_PAGE,    "R_Target page of secret" }, 
     // create buffer for validate 
     { ST_SET_RAND_NULL,  "R_Set random value (challenge) to NULL" },
     { ST_SET_BUFF_VAL,   "R_Create the validate data in temporary buffer" },
     // write to scratch with validate info 
     { ST_SELECT_COPR,    "C_Select CO-PROCESSOR port" },
     { ST_TARGET_C_MMS,   "C_Target co-processor money master secret" },
     { ST_TARGET_PAGE,    "C_Target page of secret" }, 
     { ST_WRITE_SCRATCH,  "C_Write the scratchpad" },  
     // perform a sign data page 
     { ST_SHA_SIGN_PAGE,  "C_Set SHA to sign data page" },
     { ST_COMPUTE_SHA,    "C_Compute SHA" },
     // read the MAC
     { ST_READ_SCRATCH,   "C_Read the scratchpad" },  
     { ST_SAVE_MONEY_MAC, "C_Save the money MAC" },
     // create the final account in the temp buffer
     { ST_SET_BUFF_ACCNT, "C_Set account in temporary buffer" },
     // write back to roving account page
     { ST_SELECT_ROVING,  "R_Select ROVING port" },
     { ST_TARGET_R_UDS,   "R_Target roving unique device secret" },
     { ST_TARGET_PAGE,    "R_Target page of secret" }, 
     { ST_ERASE_SCRATCH,  "R_Erase the scratchpad" }, 
     { ST_WRITE_SCRATCH,  "R_Write the scratchpad" },  
     { ST_COPY_SCRATCH,   "R_Copy the scratchpad" }, 
     { ST_FINISH,         "Finished"}}; 

// do a authenticated the money read (assume already in eCertificate)
Script VerifyMoney[] = 
    {// create packet verify
     { ST_SELECT_COPR,    "C_Select CO-PROCESSOR port" },
     // write the fixed info, balance, data page
     { ST_TARGET_C_MMS,   "C_Target co-processor money master secret" },
     { ST_TARGET_PAGE,    "C_Target page of secret" }, 
     { ST_SET_BUFF_ACCNT_FXT, "C_Set (fxt) account in temporary buffer" },
     { ST_ERASE_SCRATCH,  "C_Erase the scratchpad" }, 
     { ST_WRITE_SCRATCH,  "C_Write the scratchpad" },  
     { ST_COPY_SCRATCH,   "C_Copy the scratchpad" }, 
     // create the validation packet
     { ST_SET_RAND_NULL,  "C_Set random value (challenge) to NULL" },
     { ST_SET_BUFF_VAL,   "C_Create the validate data in temporary buffer" },
     { ST_ERASE_SCRATCH,  "C_Erase the scratchpad" }, 
     { ST_WRITE_SCRATCH,  "C_Write the scratchpad" },  
     // validate page
     { ST_SHA_VALIDATE_PAGE, "C_Set SHA to validate page" },
     { ST_COMPUTE_SHA,    "C_Compute SHA" },
     // do match with money MAC
     { ST_MATCH_SCRATCH_MONEY,"C_Perform the match scratchpad on money mac" },  
     { ST_FINISH,         "Finished"}}; 

// do a authenticated read of the roving monitary page (skip creating unique secret)
Script ReadBackAuth[] = 
    {// write random challenge to scratch of target
     { ST_SELECT_ROVING,  "R_Select ROVING port" },
     { ST_ERASE_SCRATCH,  "R_Erase the scratchpad" }, 
     { ST_TARGET_R_UDS,   "R_Target roving unique device secret" },
     { ST_TARGET_PAGE,    "R_Target page of secret" }, 
     { ST_GET_NEW_RAND,   "R_Get new random value" },
     { ST_SET_BUFF_RAND,  "R_Move random to temporary buffer" },
     { ST_WRITE_SCRATCH,  "R_Write the scratchpad" },  
     // read data page to check       
     { ST_TARGET_R_UDS,   "R_Target roving unique device secret" },
     { ST_TARGET_PAGE,    "R_Target page of secret" }, 
     { ST_READ_AUTH_PAGE, "R_Read Authenticate page" },
     { ST_MAKE_ECERT,     "R_Make eCert from read auth data" },
     // write data page to (unique secret) page of coprocessor
     { ST_SELECT_COPR,    "C_Select CO-PROCESSOR port" },
     { ST_ERASE_SCRATCH,  "C_Erase the scratchpad" }, 
     { ST_TARGET_C_UDS,   "C_Target co-processor unique device secret" },
     { ST_TARGET_PAGE,    "C_Target page of secret" }, 
     { ST_WRITE_SCRATCH,  "C_Write the scratchpad" },  
     { ST_COPY_SCRATCH,   "C_Copy the scratchpad" }, 
     // build the temp buffer with counter / rom / challenge
     { ST_SELECT_ROVING,  "R_Select ROVING port" },
     { ST_TARGET_R_UDS,   "R_Target roving unique device secret" },
     { ST_TARGET_PAGE,    "R_Target page of secret" }, 
     { ST_SET_BUFF_VAL,   "R_Create the validate data in temporary buffer" },
     // write validate to scratch of coprocessor
     { ST_SELECT_COPR,    "C_Select CO-PROCESSOR port" },
     { ST_TARGET_C_UDS,   "C_Target co-processor unique device secret" },
     { ST_TARGET_PAGE,    "C_Target page of secret" }, 
     { ST_WRITE_SCRATCH,  "C_Write the scratchpad" },  
     { ST_SHA_VALIDATE_PAGE, "C_Set SHA to validate page" },
     { ST_COMPUTE_SHA,    "C_Compute SHA" },
     // read scratch from token
     { ST_SELECT_ROVING,  "R_Select ROVING port" },
     { ST_READ_SCRATCH,   "R_Read the scratchpad" },  
     { ST_SAVE_AUTH_MAC,  "R_Save the authenticate MAC" },
     // do match scratchpad    
     { ST_SELECT_COPR,    "C_Select CO-PROCESSOR port" },
     { ST_MATCH_SCRATCH_AUTH,"C_Perform the match scratchpad on auth mac" },  
     { ST_FINISH,         "Finished"}}; 

#else

extern Script Verify1Wire[];
extern Script FindRoving[];
extern Script RovingIsPresent[];
extern Script FindCoprocessor[];
extern Script CreateAuthSecret[];
extern Script CreateMoneySecret[];
extern Script CreateUniqueDeviceSecret[];
extern Script ReadAuth[];
extern Script WriteMoney[];
extern Script VerifyMoney[];
extern Script ReadBackAuth[];
extern Script OpenSession[];
extern Script CloseSession[];
extern Script WritePage[];
extern Script SelectCoProcessor[];
extern Script SelectRoving[];
extern Script FindSHA[];

extern int VERBOSE;
extern FILE *fp;
extern long session_handle[2];
extern short PortNum[2],PortType[2];
extern unsigned char state_buffer[2][5120];
extern int dev;
extern int c_mmaster_scrt, c_amaster_scrt, c_udevice_scrt;
extern int r_udevice_scrt, t_udevice_scrt; 
extern uchar copr_file[];
extern short copr_rom[];
extern uchar provider[];
extern char user_data[];

#endif

